from ruxit.api.base_plugin import RemoteBasePlugin
from ruxit.api.exceptions import AuthException, ConfigException
import requests
import logging
import time
from orionsdk import SwisClient
from requests.packages.urllib3.exceptions import InsecureRequestWarning
requests.packages.urllib3.disable_warnings(InsecureRequestWarning)

logger = logging.getLogger(__name__)

class CustomSolarWindsPluginRemote(RemoteBasePlugin):
    previousAlerts = []
    previousEvents = []
    
    def query(self, **kwargs):
        config = kwargs['config']
        user = config['auth_user']
        password = config['auth_password']
        server = config['server']
        to_monitor = config['to_monitor']
        group = config['group']
        
        if to_monitor == '':
            raise ConfigException('You must enter which objects to monitor. It is currently blank')
            
        if user == '' or password == '':
            raise ConfigException('You must enter a user and password to authenticate')
            
        if server == '':
            raise ConfigException('You must enter a hostname or IP address where your Solarwinds resides on')
            
        activeAlerts = []
        events = []

        swis = SwisClient(server, user, password)
            
        displayNames = ''
        entities = to_monitor.split(',')
        for entity in entities:
            if displayNames != '':
                displayNames += ', '
            displayNames += "'" + entity + "'"
        logger.info(f'DisplayNames: {displayNames}')
        
        sql = f'SELECT NodeID, Nodes.DisplayName, StatusInfo.StatusName as Status, DNS, SysName, Location, Contact, LastBoot, MachineType, IOSImage, IOSVersion, SysObjectID, DynamicIP, NodeCategories.DisplayName as CategoryName FROM Orion.Nodes, Orion.NodeCategories, Orion.StatusInfo WHERE Nodes.Status=StatusInfo.StatusID AND Nodes.Category=NodeCategories.CategoryID AND Nodes.DisplayName IN ({displayNames.lower()})'
        results = swis.query(sql)
        nodes = {}
        nodeIdFilter = ""
        logger.info(f'NodesQueryResult: {results}')
        for row in results['results']:
            nodeData = {}
            nodeData['Name'] = row['DisplayName']
            nodeData['Status'] = row['Status']
            nodeData['IPs'] = []
            nodeData['Alerts'] = []
            nodeData['Events'] = []
            nodeData['Metrics'] = {}
            nodeData['Properties'] = {
                'DNS': row['DNS'],
                'System name': row['SysName'],
                'Location': row['Location'],
                'Contact': row['Contact'],
                'Last boot': row['LastBoot'],
                'Machine type': row['MachineType'],
                'Software image': row['IOSImage'],
                'Software version': row['IOSVersion'],
                'SysObjectID': row['SysObjectID'],
                'Dynamic IP': 'Yes' if row['DynamicIP'] else 'No',
                'Category': row['CategoryName']}
            
            nodes[str(row['NodeID'])] = nodeData
            if nodeIdFilter != '':
                nodeIdFilter += ','
            nodeIdFilter += str(row['NodeID'])

        results = swis.query(f'SELECT NodeID, IPAddress FROM Orion.NodeIPAddresses WHERE NodeID IN ({nodeIdFilter})')
        for row in results['results']:
            nodeData = nodes[str(row['NodeID'])]
            nodeData['IPs'].append(row['IPAddress'])
            nodes[str(row['NodeID'])] = nodeData

        if config['all_alerts']:
            now = time.time()
            logger.info(f'NodeIdFilter: {nodeIdFilter}')
            sql = f'SELECT TOP 100 aa.AlertActiveID, ao.RelatedNodeId, aa.TriggeredDateTime, aa.TriggeredMessage FROM Orion.AlertActive aa JOIN Orion.AlertObjects ao ON aa.AlertObjectID = ao.AlertObjectID WHERE ao.RelatedNodeId IN ({nodeIdFilter})'
            logger.info(f'SQL: {sql}')
            results = swis.query(sql) 
            logger.info(f'Alerts: {results}')
            for row in results['results']:
                alertId = str(row['AlertActiveID'])
                activeAlerts.append({alertId: now})
                add_alert = False
                if not any(alertId in d for d in self.previousAlerts):
                    add_alert = True
                for previousAlert in self.previousAlerts:
                    if alertId in previousAlert and now-previousAlert[alertId] > 600:
                        add_alert = True
                
                if add_alert:
                    nodeData = nodes[str(row['RelatedNodeId'])]
                    alert = {}
                    alert['Message'] = row['TriggeredMessage']
                    alert['TriggeredDateTime'] = row['TriggeredDateTime']
                    nodeData['Alerts'].append(alert)
                    nodes[str(row['RelatedNodeId'])] = nodeData
                    
            results = swis.query("SELECT TOP 100 ev.EventTime, ev.EventID, ev.NetworkNode, ev.NetObjectID, ev.NetObjectType, ev.Message, ev.EventType, evType.Name AS EventTypeName " + 
                "FROM Orion.Events ev " + 
                "JOIN Orion.EventTypes evType ON ev.EventType = evType.EventType " + 
                "WHERE ev.NetObjectID IN ("+nodeIdFilter+") AND ev.EventType IN (1,10) "+
                "AND ev.EventTime > ADDSECOND(-60,GETUTCDATE())")
            logger.info(f'Events: {results}')
            for row in results['results']:
                eventId = str(row['EventID'])
                events.append({eventId: now})
                add_event = False
                if not any(eventId in d for d in self.previousEvents):
                    add_event = True
                for previousEvent in self.previousEvents:
                    if eventId in previousEvent and now-previousEvent[eventId] > 600:
                        add_event = True
                        
                if add_event:
                    nodeData = nodes[str(row['NetworkNode'])]
                    event = {}
                    event['Message'] = row['Message']
                    event['EventTime'] = row['EventTime']
                    nodeData['Events'].append(event)
                    nodes[str(row['NetworkNode'])] = nodeData
            
            self.previousAlerts = activeAlerts
            self.previousEvents = events
            
        results = swis.query(f'SELECT NodeID, CPULoad, PercentLoss, AvgResponseTime, PercentMemoryUsed FROM Orion.NodesStats WHERE NodeID IN ({nodeIdFilter})')
        for row in results['results']:
            nodeData = nodes[str(row['NodeID'])]
            nodeData['Metrics']['CPULoad'] = row['CPULoad']
            nodeData['Metrics']['PercentLoss'] = row['PercentLoss']
            nodeData['Metrics']['AvgResponseTime'] = row['AvgResponseTime']
            nodeData['Metrics']['PercentMemoryUsed'] = row['PercentMemoryUsed']
            nodes[str(row['NodeID'])] = nodeData

        g1 = self.topology_builder.create_group('Solarwinds - ' + group, 'Solarwinds - ' + group)
        for node in nodes:
            e1 = g1.create_element('Solarwinds - ' + group + ' - ' + nodes[node]['Name'], 'Solarwinds - ' + group + ' - ' + nodes[node]['Name'])
            for ip in nodes[node]['IPs']:
                e1.add_endpoint(ip)
            for property in nodes[node]['Properties']:
                e1.report_property(property, str(nodes[node]['Properties'][property]))
            e1.state_metric('status', nodes[node]['Status'])
            e1.absolute(key='cpuload', value=nodes[node]['Metrics']['CPULoad'])
            e1.absolute(key='percentloss', value=nodes[node]['Metrics']['PercentLoss'])
            e1.absolute(key='responsetime', value=nodes[node]['Metrics']['AvgResponseTime'])
            e1.absolute(key='percentmemoryused', value=nodes[node]['Metrics']['PercentMemoryUsed'])
            logger.info(f'Alerts to report back: {nodes[node]["Alerts"]}')
            for alert in nodes[node]['Alerts']:
                if alert['Message'] is not None and alert['Message'] is not '':
                    e1.report_availability_event(f"{nodes[node]['Name']}: {alert['Message']}", f"{nodes[node]['Name']}: {alert['Message']}", {'TriggeredDateTime': alert['TriggeredDateTime']})
                    logger.info(f'Sent alert to {nodes[node]["Name"]} with message: {alert["Message"]}')
                
            for event in nodes[node]['Events']:
                if event['Message'] is not None:
                    e1.report_custom_info_event(f"{nodes[node]['Name']}: {event['Message']}", f"{nodes[node]['Name']}: {event['Message']}", {'EventTime': event['EventTime']})
                    logger.info(f'Sent event to {nodes[node]["Name"]} with message: {event["Message"]}')
